/*
  Mod projektu "Identyfikacja"

  Plik zawiera:
     implementacj klasy RPN.
     implementacj klasy Model.
  
  Copyrights (c) 2008 Micha Kowalik
  
  e-mail: maf-michal@gmail.com

  LICENCJA:
  Kod mona
    - zastosowa w swoim programie pod warunkiem, e poinformuje si o tym autora i umieci o nim informacje,
    - poprawia/uleprza - chciabym by skontaktowano si ze mn w tej sprawie i napisno co i jak,
    - oddawa na zaliczenie o ile wyle si e-mail do autora z wielkimi podziekowaniami (odele adres na ktry bdziesz mg(a) wysac piwo).
*/

#include "identification.h"

#include <cmath>
#include <istream>
#include <sstream>

std::ostream & operator << (std::ostream & Out, const RPN & rpn)
{
	Out << '{';
	for (unsigned i=0; i<rpn._heap.size(); i++) {
		switch (rpn._heap[i].Type())
		{
		case RPN::Element::TOPERATOR:
			Out << rpn._heap[i].Op();
			break;
		case RPN::Element::TNUMBER:
			Out << rpn._heap[i].Num();
			break;
		case RPN::Element::TVALUE:
			Out << 'u' << (rpn._heap[i].Id()+1);
			break;
		} Out << ' ';
	}
	return Out << '}';
}

unsigned RPN::Element::Prior(const char & ch)
{
	switch (ch)
	{
		case '+':case '-': return 1;
		case '*':case '/': return 2;
		case '^':          return 3;
		default:           return 0;
	}
}

RPN::Element & RPN::Element::operator = (const RPN::Element & elem)
{
	_type = elem._type;
	switch (_type)
	{
	case TNUMBER: _num = elem._num; break;
	default:      _id = elem._id;   break;
	}
	return *this;
}

double RPN::Solve(const double * arr) const
{
	std::vector<double> heap;

	for (unsigned i=0; i<_heap.size(); i++) switch (_heap[i].Type())
	{
	case Element::TOPERATOR:
		switch (_heap[i].Op())
		{
		case '+':
			{
				unsigned id = heap.size()-1;
				double b = heap[id--]; heap.pop_back();
				heap[id] = heap[id] + b;
			}
			break;
		case '-':
			{
				unsigned id = heap.size()-1;
				double b = heap[id--]; heap.pop_back();
				heap[id] = heap[id] - b;
			}
			break;
		case '*':
			{
				unsigned id = heap.size()-1;
				double b = heap[id--]; heap.pop_back();
				heap[id] = heap[id] * b;
			}
			break;
		case '/':
			{
				unsigned id = heap.size()-1;
				double b = heap[id--]; heap.pop_back();
				heap[id] = heap[id] / b;
			}
			break;
		case '^':
			{
				unsigned id = heap.size()-1;
				double b = heap[id--]; heap.pop_back();
				heap[id] = std::pow(heap[id],b);
			}
			break;
		default: throw ERR_NIEZNANYOPERATOR;
		}
		break;
	case RPN::Element::TNUMBER:
		heap.push_back(_heap[i].Num());
		break;
	default:
		if (!arr) throw ERR_BRAKPARAMETROW;
		heap.push_back(arr[_heap[i].Id()]);
		break;
	}

	return heap[0];
}

bool RPN::Input(std::istream & input, bool separate)
{
	std::vector<char> heap;

	while (!input.eof() && input.get() <= 32);
		input.unget();

	unsigned nawias = 0;

	char ch = 0;
	while (!input.eof())
	{
		ch = (char)input.get();
		switch (ch)
		{
		case 'u':case 'U':
			{
				unsigned i = 1;
				input >> i;
				input.clear();
				AddVar(i-1);
			}
			break;
		case '(':
			{
				nawias++;
				heap.push_back(ch);
			}
			break;
		case ')':
			{
				nawias--;
				unsigned i = heap.size();
				while (i--)
				{
					if (heap[i] == '(') {
						heap.pop_back();
						goto _endLoop1;
					}
					AddOP(heap[i]);
					heap.pop_back();
				}
				throw ERR_BLADNAWIASU;
				_endLoop1:;
			}
			break;
		case '+':
			if (separate && !nawias)
				goto _end;
		case '*': case '/': case '^': case '-':
			{
				unsigned prior = RPN::Element::Prior(ch);
				unsigned i = heap.size();
				while (i--)
				{
					if (RPN::Element::Prior(heap[i]) >= prior) {
						AddOP(heap[i]);
						heap.pop_back();
					} else break;
				}
				heap.push_back(ch);
			}
			break;
		case '\n': goto _end;
		default:
			if (ch > 32)
			{
				input.unget();
				double num; input >> num;
				if (input.bad()) throw ERR_NIEZNANYZNAK;
				AddConst(num);
			}
			break;
		}
	} _end:

	unsigned i = heap.size();
	while (i--)
	{
		if (heap[i] != '(')
			AddOP(heap[i]);
		heap.pop_back();
	}

	return ch != '\n';
}

RPN & RPN::operator = (const RPN & rpn)
{
	_heap = rpn._heap;
	return *this;
}

RPN & RPN::operator = (const double x)
{
	_heap.clear();
	AddConst(x);
	return *this;
}

void Model::LoadUY(std::istream & source)
{
	if (source.bad()) throw ERR_1;

	if (source == std::cin)
		std::cout << "Ilosc pomiarow: ";
	unsigned n; source >> n;
	if (source.bad()) throw ERR_2;

	if (source == std::cin)
		std::cout << "Ilosc wejsc: ";
	unsigned x; source >> x;
	if (source.bad()) throw ERR_3;

	_u.Resize(n,x);
	for (unsigned i=0; i<x; i++) {
		if (source == std::cin)
			std::cout << "u" << (i+1) << ": ";
		for (unsigned j=0; j<n; j++)
			source >> _u[j][i];
	}

	if (source == std::cin)
		std::cout << "Ilosc wyjsc: ";
	source >> x;
	if (source.bad()) throw ERR_4;

	_y.Resize(x,n);
	for (unsigned i=0; i<x; i++) {
		if (source == std::cin)
			std::cout << "y" << (i+1) << ": ";
		for (unsigned j=0; j<n; j++)
			source >> _y[i][j];
	}
}

void Model::LoadModel(std::istream & source)
{
    if (source.bad()) throw ERR_1;

	if (source == std::cin)
		std::cout << "Dl. najdluzszego rownania: ";
	unsigned x; source >> x;
	if (source.bad()) throw ERR_5;

	_eq.Clear();
	_eq.Resize(x, _y.Rows());

	for (unsigned i=0; i<_y.Rows(); i++) {

	    if (source == std::cin)
            std::cout << 'y' << (i+1) << " = ";

	    unsigned j=0;
		while (j<x && _eq[j++][i].Input(source));
		while (j<x) _eq[j++][i] = 0;
	}

	if (source == std::cin)
		std::cout << std::endl;
}

void Model::Import(std::istream & source)
{
	LoadUY(source);
	LoadModel(source);
}

Matrix<double> Model::Identify()
{
	unsigned m = _eq.Rows(), n = _y.Rows();

	Matrix<double> fI(m, m), b(m, n),
	               f(m, n), ft(n, m);

	fI.O(); b.O();

	for (unsigned i=0; i<_u.Rows(); i++)
	{
		//oblicza fi(ui)
		for (unsigned x=0; x<m; x++)
			for (unsigned y=0; y<n; y++)
				f[x][y] = _eq[x][y].Solve(_u[i]);
		//wypelnia fi^T(ui)
		for (unsigned x=0; x<m; x++)
			for (unsigned y=0; y<n; y++)
				ft[y][x] = f[x][y];
		//oblicza yi*fi(ui)
		for (unsigned x=0; x<m; x++)
			for (unsigned y=0; y<n; y++)
				b[x][y] += (_y[y][i]*f[x][y]);
		//
		fI += (f*ft);
	}

	//oblicza fi(u)*fi^T(u)*y*f(u)
	//fi == Sigma(fi(ui)*fi^T(ui))
	//b == Sigma(yi*f(ui))
	(fI^=-1)*=b;

	return fI;
}
