/*
  Plik nagwkowy projektu "Identyfikacja"

  Plik zawiera:
     klas RPN - reprezentuje rwnanie RPN.
     kals Model - reprezentuje klas mogc rozpoznawa parametry modelu.
  
  Copyrights (c) 2008 Micha Kowalik
  
  e-mail: maf-michal@gmail.com

  LICENCJA:
  Kod mona
    - zastosowa w swoim programie pod warunkiem, e poinformuje si o tym autora i umieci o nim informacje,
    - poprawia/uleprza - chciabym by skontaktowano si ze mn w tej sprawie i napisno co i jak,
    - oddawa na zaliczenie o ile wyle si e-mail do autora z wielkimi podziekowaniami (odele adres na ktry bdziesz mg(a) wysac piwo).
*/

#ifndef identificationH
#define identificationH

#include <iostream>
#include <vector>
#include <fstream>

#include "matrix.h"

class RPN
{
public:
	enum
	{
		ERR_BLADNAWIASU      = 11,
		ERR_NIEZNANYZNAK     = 12,
		ERR_BRAKPARAMETROW   = 13,
		ERR_NIEZNANYOPERATOR = 14
	};
	class Element
	{
	public:
		enum
		{
			TOPERATOR = 21,
			TNUMBER   = 22,
			TVALUE    = 23
		};
	private:
		unsigned char _type;
		union
		{
			char     _op;
			unsigned _id;
			double   _num;
		};
	public:
		Element(const double num=0): _type(TNUMBER), _num(num) {}
		Element(const char op):      _type(TOPERATOR), _op(op) {}
		Element(const unsigned id):  _type(TVALUE), _id(id)    {}

		double          Num()   const {return _num;}
		unsigned        Id()    const {return _id;}
		unsigned char   Type()  const {return _type;}

		unsigned        Prior() const {return _type==TOPERATOR?Prior(_op):0;}
		char            Op()    const {return _op;}
		static unsigned Prior(const char & ch);

		Element & operator = (const Element & elem);
	};
private:
	std::vector<Element> _heap;
public:
	RPN(): _heap() {}
	RPN(const RPN & rpn): _heap(rpn._heap) {}

	const Element & operator [] (const size_t id) const {return _heap[id];}
	unsigned size() const {return _heap.size();}

	double Solve(const double * arr=NULL) const;

	void AddConst(const double num) {_heap.push_back(num);}
	void AddVar(const unsigned id)  {_heap.push_back(id);}
	void AddOP(const char op)       {_heap.push_back(op);}

	RPN & operator = (const RPN & rpn);
	RPN & operator = (const double x);

	bool Input(std::istream & input, bool separate = true);

	friend std::ostream & operator << (std::ostream &, const RPN &);
};

std::ostream & operator << (std::ostream & Out, const RPN & rpn);

class Model
{
public:
	enum
	{
		ERR_1 = 31,
		ERR_2 = 32,
		ERR_3 = 33,
		ERR_4 = 34,
		ERR_5 = 35,
		ERR_6 = 36
	};
private:
	Matrix<double> _y;
	Matrix<double> _u;
	Matrix<RPN>    _eq;
public:
	Model(): _y(), _u(), _eq() {}
	explicit Model(const char * path) {std::ifstream f(path); Import(f);}
	explicit Model(std::istream & input) {Import(input);}

	void LoadUY(std::istream & f = std::cin);
	void LoadModel(std::istream & f = std::cin);
	void Import(std::istream & f = std::cin);

	Matrix<double> Identify();

	unsigned n() const {return _y.Columns();}

	const Matrix<double> & u()     const {return _u;}
	const Matrix<double> & y()     const {return _y;}
	const Matrix<RPN>    & model() const {return _eq;}
};

#endif
