/*
  Szablon macierzy jedno lub dwuwymiarowej dowlonego typu

  Plik zawiera:
    - szablon macierzy (Matrix)
  
  Copyrights (c) 2008 Micha Kowalik
  
  e-mail: maf-michal@gmail.com

  LICENCJA:
  Kod mona
    - zastosowa w swoim programie pod warunkiem, e poinformuje si o tym autora i umieci o nim informacje,
    - poprawia/uleprza - chciabym by skontaktowano si ze mn w tej sprawie i napisno co i jak,
    - oddawa na zaliczenie o ile wyle si e-mail do autora z wielkimi podziekowaniami (odele adres na ktry bdziesz mg(a) wysac piwo).
*/

#ifndef matrixH
#define matrixH

#include <iostream>
#include <cfloat>
#include <ctime>
#include <cmath>
#include <cstring>
#include <cstdlib>

template<class Type> class Matrix
{
	#define MATRIX_ERR1	1
	#define MATRIX_ERR2	2
	#define MATRIX_ERR3	3
	#define MATRIX_ERR4	4
	#define MATRIX_ERR5	5
	#define MATRIX_ERR6	6
	#define MATRIX_ZERO	(2.0*DBL_EPSILON)
private:
	Type * _arr;
	unsigned _m;
	unsigned _n;
	static bool _resize(Type * &_arr, unsigned m, unsigned n);
	static void _multi(Type * &_newArr, const Matrix<Type> & L, const Matrix<Type> & P);
public:
	explicit Matrix(unsigned m=0, unsigned n=0);
	Matrix(const Matrix<Type> &M);
	virtual ~Matrix() {Clear();}

	void I();
	void O();
	void Clear();
	void Resize(unsigned m, unsigned n);
	void Print(std::ostream & Out = std::cout) const;
	void Input(std::istream & In = std::cin);
	void Reverse();
	bool Squered() const {return _m && _m == _n;}
	Type Gauss(Matrix<Type> * M = NULL);

	unsigned Columns() const {return _n;}
	unsigned Rows() const {return _m;}

	void ExchangeRow(unsigned Index1, unsigned Index2);
	void ExchangeColumn(unsigned Index1, unsigned Index2);

	Matrix<Type> & operator += (const Matrix<Type> &P);
	Matrix<Type> & operator -= (const Matrix<Type> &P);
	Matrix<Type> & operator ^= (int N);
	Matrix<Type> & operator *= (const Matrix<Type> &P);

	Matrix<Type> operator + (const Matrix<Type> &P) {Matrix<Type> M(*this); return M += P;}
	Matrix<Type> operator - (const Matrix<Type> &P) {Matrix<Type> M(*this); return M -= P;}
	Matrix<Type> operator ^ (const int N)           {Matrix<Type> M(*this); return M ^= N;}
	Matrix<Type> operator * (const Matrix<Type> &P);

	bool operator == (const Matrix<Type> &P) const;
	Matrix<Type> & operator = (const Matrix<Type> &P);

	Type * operator [] (const size_t Index) {return _arr+Index*_n;}
	Type const * operator [] (const size_t Index) const {return _arr+Index*_n;}
};

template<class Type> inline std::ostream & operator << (std::ostream & Out, const Matrix<Type> &M)
{
	M.Print(Out);
	return Out;
}

template<class Type> inline std::istream & operator >> (std::istream & In, Matrix<Type> &M)
{
	M.Input(In);
	return In;
}

template<class Type> Matrix<Type>::Matrix(const Matrix<Type> &M): _arr(NULL), _m(0), _n(0)
{
	if (_resize(_arr, M._m, M._n)) {
		_m = M._m; _n = M._n;
		std::memcpy(_arr, M._arr, _m*_n*sizeof(Type));
	}
}

template<class Type> Matrix<Type>::Matrix(unsigned m, unsigned n): _arr(NULL), _m(0), _n(0)
{
	if (_resize(_arr, m, n)) {
		_m = m; _n = n;
	}
}

template<class Type> bool Matrix<Type>::_resize(Type * &_arr, unsigned m, unsigned n)
{
	_arr = NULL; if (m && n)
		_arr = new Type [m*n];
	return _arr!=NULL;
}

template<class Type> void Matrix<Type>::_multi(Type * &_newArr, const Matrix<Type> & L, const Matrix<Type> & P)
{
	if (L._n != P._m) throw MATRIX_ERR4;

	if (!_resize(_newArr, L._m, P._n))
		throw MATRIX_ERR1;

	for (unsigned i=0; i<L._m; i++)
		for (unsigned j=0; j<P._n; j++) {
			Type tmp = 0;
			for (unsigned x=0; x<L._n; x++)
				tmp += (L._arr[i*L._n+x]*P._arr[x*P._n+j]);
			_newArr[i*P._n+j] = tmp;
		}
}

template<class Type> void Matrix<Type>::Clear()
{
	delete [] _arr;
	_arr = NULL;
	_m = 0; _n = 0;
}

template<class Type> void Matrix<Type>::Resize(unsigned m, unsigned n)
{
	Type * _newArr;
	if (_resize(_newArr, m, n))
	{
		memset(_newArr, 0, m*n*sizeof(Type));
		unsigned w=n>_n?_n:n, h=m>_m?_m:m;
		for (unsigned i=0; i<h; i++)
			memcpy(_newArr+i*n, _arr+i*_n, w*sizeof(Type));
		_arr = _newArr; _m = m; _n = n;
	} else throw MATRIX_ERR1;
}

template<class Type> void Matrix<Type>::I()
{
	O();
	for (unsigned i=0; i<_m*_n; i+=(_m+1))
		_arr[i] = 1;
}

template<class Type> void Matrix<Type>::O()
{
	memset(_arr, 0, _m*_n*sizeof(Type));
}

template<class Type> void Matrix<Type>::Print(std::ostream & Out) const
{
	unsigned w = Out.width(); w=w>0?w:1;
	for (unsigned i=0; i<_m; i++) {Out << '|';
		for (unsigned j=0; j<_n; j++) {Out << ' ';
			Out.width(w); Out << _arr[i*_n+j];
		} Out << ' ' << '|' << std::endl;
	}
}

template<class Type> void Matrix<Type>::Input(std::istream & In)
{
	Clear(); bool alert = (In == std::cin);
	if (alert) std::cout << "Podaj rozmiary: ";
	In >> _m >> _n; if (!_m || !_n) {_m=0;_n=0;return;}

	if (!_resize(_arr, _m, _n)) {
		_m = 0; _n = 0; throw MATRIX_ERR1;
	}

	for (unsigned i=0; i<_m*_n; i++) {
		if (alert)
			std::cout << "Macierz[" << (i/_n+1) << "][" << (i%_n+1) << "]=";
		In >> _arr[i];
	}
}

template<class Type> void Matrix<Type>::Reverse()
{
	if (!Squered()) throw MATRIX_ERR5;
	Matrix M(*this); I();
	M.Gauss(this);
	for (unsigned i=0; i<_m; i++) {
		Type tmp = M._arr[i*_n+i];
		if (std::abs(tmp) < MATRIX_ZERO)
			throw MATRIX_ERR6;
		M._arr[i*_n+i] = 1;
		for (unsigned j=i+1; j<_n; j++)
			M._arr[i*_n+j] /= tmp;
		for (unsigned j=0; j<_n; j++)
			_arr[i*_n+j] /= tmp;
	}
	for (unsigned i=_n-1; i>0; i--) {
		for (unsigned j=i-1; ; j--) {
			Type tmp = M._arr[j*_n+i]; M._arr[j*_n+i] = 0;
			for (unsigned x=0; x<_n; x++)
				_arr[j*_n+x] -= (_arr[i*_n+x]*tmp);
			if (j==0) break;
		}
	}
}

template<class Type> Type Matrix<Type>::Gauss(Matrix<Type> * M)
{
	Type w = 1;

	for (unsigned a=0; a<_m; a++) {unsigned x;
		if (_arr[a*_n+a] == 0) {
			for (x=a; x<_m; x++)
				if (_arr[x*_n+a]) break;
			if (x >= _m) continue;
			ExchangeRow(a+1, x+1); w *= -1;
			if (M) M->ExchangeRow(a+1, x+1);
		}
		for (x=a+1; x<_m; x++) {
			if (_arr[x*_n+a]) {
				Type tmp = _arr[x*_n+a] / _arr[a*_n+a];
				_arr[x*_n+a] = 0;
				for (unsigned i=a+1; i<_n; i++)
					_arr[x*_n+i] -= (_arr[a*_n+i]*tmp);
				if (M) for (unsigned i=0; i<_n; i++)
					M->_arr[x*_n+i] -= (M->_arr[a*_n+i]*tmp);
			}
		}
	}

	return w;
}

template<class Type> void Matrix<Type>::ExchangeRow(unsigned Index1, unsigned Index2)
{
	if (Index1 == Index2) return;
	if (!Index1 || Index1 > _m || !Index2 || Index2 > _m)
		throw MATRIX_ERR2; Index1--; Index2--;

	Type * arr = new Type[_n];
	if (!arr) throw MATRIX_ERR1;

	memcpy(arr, _arr+Index1*_n, _n*sizeof(Type));
	memcpy(_arr+Index1*_n, _arr+Index2*_n, _n*sizeof(Type));
	memcpy(_arr+Index2*_n, arr, _n*sizeof(Type));

	delete [] arr;
}

template<class Type> void Matrix<Type>::ExchangeColumn(unsigned Index1, unsigned Index2)
{
	if (Index1 == Index2) return;
	if (!Index1 || Index1 > _n || !Index2 || Index2 > _n)
		throw MATRIX_ERR2; Index1--; Index2--;

	Type * arr = new Type[_m];
	if (!arr) throw MATRIX_ERR1;

	unsigned i, x;
	for (i=Index1, x=0; i<_m*_n; i+=_n, x++) arr[x] = _arr[i];
	for (i=Index1, x=Index2; i<_m*_n; i+=_n, x+=_n) _arr[i] = _arr[x];
	for (i=Index2, x=0; i<_m*_n; i+=_n, x++) _arr[i] = arr[x];

	delete [] arr;
}

template<class Type> Matrix<Type> & Matrix<Type>::operator += (const Matrix<Type> &P)
{
	if (_n != P._n || _m != P._m)
		throw MATRIX_ERR3;
	for (unsigned i=0; i<_m*_n; i++)
		_arr[i] += P._arr[i];
	return *this;
}

template<class Type> Matrix<Type> & Matrix<Type>::operator -= (const Matrix<Type> &P)
{
	if (_n != P._n || _m != P._m)
		throw MATRIX_ERR3;
	for (unsigned i=0; i<_m*_n; i++)
		_arr[i] -= P._arr[i];
	return *this;
}

template<class Type> Matrix<Type> & Matrix<Type>::operator *= (const Matrix<Type> &P)
{
	Type * _newArr;
	_multi(_newArr, *this, P);
	_n = P._n; delete [] _arr;
	_arr = _newArr;
	return *this;
}

template<class Type> Matrix<Type> Matrix<Type>::operator * (const Matrix<Type> &P)
{
	Matrix<Type> M;
	_multi(M._arr, *this, P);
	M._m = _m; M._n = P._n;
	return M;
}

template<class Type> Matrix<Type> & Matrix<Type>::operator ^= (int N)
{
	if (N < 0) {N *= -1; Reverse();}
	Matrix<Type> M(_m, _n); M.I();

	while (N) {
		if (N % 2) M *= *this;
		*this *= *this; N /= 2;
	}
	*this = M;

	return *this;
}

template<class Type> bool Matrix<Type>::operator == (const Matrix<Type> &P) const
{
	if (_m != P._m || _n != P._n)
		return false;
	for (unsigned i=0; i<_m*_n; i++)
		if (_arr[i] != P._arr[i])
			return false;
	return true;
}

template<class Type> Matrix<Type> & Matrix<Type>::operator = (const Matrix<Type> &P)
{
	if (_m != P._m || _n != P._n) {Clear();
        if (!_resize(_arr, P._m, P._m))
            throw MATRIX_ERR1;
        _m = P._m; _n = P._n;
	}
	memcpy(_arr, P._arr, _m*_n*sizeof(Type));
	return *this;
}

#endif
